@@ -22,6 +22,8 @@ import os
 import unittest
 import shutil
 import tempfile
+import sys
+import stat
 
 import in_toto.settings
 import in_toto.exceptions
@@ -30,13 +32,18 @@ from in_toto.exceptions import SignatureVerificationError
 from in_toto.runlib import (in_toto_run, in_toto_record_start,
     in_toto_record_stop, record_artifacts_as_dict, _apply_exclude_patterns,
     _hash_artifact)
-from in_toto.util import (generate_and_write_rsa_keypair,
-    prompt_import_rsa_key_from_file)
+from securesystemslib.interface import (
+    generate_and_write_unencrypted_rsa_keypair,
+    import_rsa_privatekey_from_file,
+    import_rsa_publickey_from_file)
 from in_toto.models.link import UNFINISHED_FILENAME_FORMAT, FILENAME_FORMAT
 
 import securesystemslib.formats
 import securesystemslib.exceptions
 
+from tests.common import TmpDirMixin
+
+
 class Test_ApplyExcludePatterns(unittest.TestCase):
   """Test _apply_exclude_patterns(names, exclude_patterns) """
 
@@ -83,7 +90,7 @@ class Test_ApplyExcludePatterns(unittest.TestCase):
     self.assertListEqual(result, expected)
 
 
-class TestRecordArtifactsAsDict(unittest.TestCase):
+class TestRecordArtifactsAsDict(unittest.TestCase, TmpDirMixin):
   """Test record_artifacts_as_dict(artifacts). """
 
   @classmethod
@@ -98,18 +105,13 @@ class TestRecordArtifactsAsDict(unittest.TestCase):
             `-- foosubsub
     """
 
-    self.working_dir = os.getcwd()
-
     # Backup and clear user set exclude patterns and base path
     self.artifact_exclude_orig = in_toto.settings.ARTIFACT_EXCLUDE_PATTERNS
     self.artifact_base_path_orig = in_toto.settings.ARTIFACT_BASE_PATH
     in_toto.settings.ARTIFACT_EXCLUDE_PATTERNS = []
     in_toto.settings.ARTIFACT_BASE_PATH = None
 
-    # mkdtemp uses $TMPDIR, which might contain a symlink
-    # but we want the absolute location instead
-    self.test_dir = os.path.realpath(tempfile.mkdtemp())
-    os.chdir(self.test_dir)
+    self.set_up_test_dir()
 
     # Create files on 3 levels
     os.mkdir("subdir")
@@ -126,8 +128,7 @@ class TestRecordArtifactsAsDict(unittest.TestCase):
   @classmethod
   def tearDownClass(self):
     """Change back to working dir, remove temp directory, restore settings. """
-    os.chdir(self.working_dir)
-    shutil.rmtree(self.test_dir)
+    self.tear_down_test_dir()
     in_toto.settings.ARTIFACT_EXCLUDE_PATTERNS = self.artifact_exclude_orig
     in_toto.settings.ARTIFACT_BASE_PATH = self.artifact_base_path_orig
 
@@ -280,6 +281,14 @@ class TestRecordArtifactsAsDict(unittest.TestCase):
     self.assertListEqual(sorted(list(artifacts_dict.keys())),
       sorted(self.full_file_path_list))
 
+  @staticmethod
+  def _raise_win_dev_mode_error():
+    """If the platform is Windows, raises an error that asks the user if
+    developer mode is activated."""
+    if os.name == "nt":
+      raise IOError("Developer mode is required to work with symlinks on "
+          "Windows. Is it enabled?")
+
 
   @unittest.skipIf("symlink" not in os.__dict__, "symlink is not supported in this platform")
   def test_record_symlinked_files(self):
@@ -295,7 +304,11 @@ class TestRecordArtifactsAsDict(unittest.TestCase):
     for pair in link_pairs:
       # We only use the basename of the file (source) as it is on the same
       # level as the link (target)
-      os.symlink(os.path.basename(pair[0]), pair[1])
+      try:
+        os.symlink(os.path.basename(pair[0]), pair[1])
+      except IOError:
+        TestRecordArtifactsAsDict._raise_win_dev_mode_error()
+        raise
 
     # Record files and linked files
     # follow_symlink_dirs does not make a difference as it only concerns linked dirs
@@ -328,7 +341,11 @@ class TestRecordArtifactsAsDict(unittest.TestCase):
 
     # Create dead links
     for link in links:
-      os.symlink("does/not/exist", link)
+      try:
+        os.symlink("does/not/exist", link)
+      except IOError:
+        TestRecordArtifactsAsDict._raise_win_dev_mode_error()
+        raise
 
     # Record files without dead links
     # follow_symlink_dirs does not make a difference as it only concerns linked dirs
@@ -348,8 +365,12 @@ class TestRecordArtifactsAsDict(unittest.TestCase):
   def test_record_follow_symlinked_directories(self):
     """Record files in symlinked dirs if follow_symlink_dirs is True. """
 
-    # Link to subdir
-    os.symlink("subdir", "subdir_link")
+    try:
+      # Link to subdir
+      os.symlink("subdir", "subdir_link")
+    except IOError:
+      TestRecordArtifactsAsDict._raise_win_dev_mode_error()
+      raise
 
     link_pairs = [
       ("subdir/foosub1", "subdir_link/foosub1"),
@@ -367,7 +388,6 @@ class TestRecordArtifactsAsDict(unittest.TestCase):
     for pair in link_pairs:
       self.assertDictEqual(artifacts_dict[pair[0]], artifacts_dict[pair[1]])
 
-
     # Record with follow_symlink_dirs FALSE (default)
     artifacts_dict = record_artifacts_as_dict(["."])
     self.assertListEqual(sorted(list(artifacts_dict.keys())),
@@ -424,11 +444,34 @@ class TestRecordArtifactsAsDict(unittest.TestCase):
 
   def test_hash_artifact_passing_algorithm(self):
     """Test _hash_artifact passing hash algorithm. """
-    os.chdir(self.test_dir)
     self.assertTrue("sha256" in list(_hash_artifact("foo", ["sha256"]).keys()))
 
 
-class TestInTotoRun(unittest.TestCase):
+class TestLinkCmdExecTimeoutSetting(unittest.TestCase):
+  """Tests LINK_CMD_EXEC_TIMEOUT setting in settings.py file. """
+
+  def test_timeout_setting(self):
+    # Save the old timeout and make sure it's saved properly
+    timeout_old = in_toto.settings.LINK_CMD_EXEC_TIMEOUT
+
+    # Modify timeout
+    in_toto.settings.LINK_CMD_EXEC_TIMEOUT = 0.1
+
+    # check if exception is raised
+    with self.assertRaises(securesystemslib.process.subprocess.TimeoutExpired):
+      # Call execute_link to see if new timeout is respected
+      in_toto.runlib.execute_link([sys.executable, '-c', 'while True: pass'], True)
+
+    # check if exception is raised
+    with self.assertRaises(securesystemslib.process.subprocess.TimeoutExpired):
+      # Call execute_link to see if new timeout is respected
+      in_toto.runlib.execute_link([sys.executable, '-c', 'while True: pass'], False)
+
+    # Restore original timeout
+    in_toto.settings.LINK_CMD_EXEC_TIMEOUT = timeout_old
+
+
+class TestInTotoRun(unittest.TestCase, TmpDirMixin):
   """"
   Tests runlib.in_toto_run() with different arguments
 
@@ -444,26 +487,21 @@ class TestInTotoRun(unittest.TestCase):
   def setUpClass(self):
     """Create and change into temporary directory, generate key pair and dummy
     material, read key pair. """
-
-    self.working_dir = os.getcwd()
-
-    self.test_dir = tempfile.mkdtemp()
-    os.chdir(self.test_dir)
+    self.set_up_test_dir()
 
     self.step_name = "test_step"
     self.key_path = "test_key"
-    generate_and_write_rsa_keypair(self.key_path)
-    self.key = prompt_import_rsa_key_from_file(self.key_path)
-    self.key_pub = prompt_import_rsa_key_from_file(self.key_path + ".pub")
+    generate_and_write_unencrypted_rsa_keypair(self.key_path)
+    self.key = import_rsa_privatekey_from_file(self.key_path)
+    self.key_pub = import_rsa_publickey_from_file(self.key_path + ".pub")
 
     self.test_artifact = "test_artifact"
     open(self.test_artifact, "w").close()
 
   @classmethod
   def tearDownClass(self):
-    """Change back to initial working dir and remove temp test directory. """
-    os.chdir(self.working_dir)
-    shutil.rmtree(self.test_dir)
+    self.tear_down_test_dir()
+
 
   def tearDown(self):
     """Remove link file if it was created. """
@@ -510,6 +548,35 @@ class TestInTotoRun(unittest.TestCase):
         FILENAME_FORMAT.format(step_name=self.step_name, keyid=self.key["keyid"]))
     self.assertEqual(repr(link), repr(link_dump))
 
+  def test_in_toto_run_with_metadata_directory(self):
+    """Successfully run with metadata directory,
+     compare dumped link is equal to returned link"""
+    tmp_dir = os.path.realpath(tempfile.mkdtemp(dir=os.getcwd()))
+    link = in_toto_run(self.step_name, [self.test_artifact],
+        [self.test_artifact], ["python", "--version"], True, self.key,
+        metadata_directory=tmp_dir)
+    file_path = os.path.join(tmp_dir, FILENAME_FORMAT.format(step_name=self.step_name,
+        keyid=self.key["keyid"]))
+    link_dump = Metablock.load(file_path)
+    self.assertEqual(repr(link), repr(link_dump))
+
+  def test_in_toto_run_compare_with_and_without_metadata_directory(self):
+    """Successfully run with and without metadata directory,
+     compare the signed is equal"""
+    tmp_dir = os.path.realpath(tempfile.mkdtemp(dir=os.getcwd()))
+    in_toto_run(self.step_name, [self.test_artifact], [self.test_artifact],
+        ["python", "--version"], True, self.key, metadata_directory=tmp_dir)
+    file_path = os.path.join(tmp_dir, FILENAME_FORMAT.format(step_name=self.step_name,
+        keyid=self.key["keyid"]))
+    link_dump_with_md = Metablock.load(file_path)
+
+    in_toto_run(self.step_name, [self.test_artifact], [self.test_artifact],
+        ["python", "--version"], True, self.key)
+    link_dump_without_md = Metablock.load(
+        FILENAME_FORMAT.format(step_name=self.step_name, keyid=self.key["keyid"]))
+    self.assertEqual(repr(link_dump_with_md.signed),
+        repr(link_dump_without_md.signed))
+
   def test_in_toto_run_verify_recorded_artifacts(self):
     """Successfully run, verify properly recorded artifacts. """
     link = in_toto_run(self.step_name, [self.test_artifact],
@@ -563,22 +630,53 @@ class TestInTotoRun(unittest.TestCase):
       in_toto_run(self.step_name, None, None,
           ["python", "--version"], True, self.key_pub)
 
+  def test_nonexistent_directory(self):
+    """Fail run, passed metadata_directory not exist. """
+    expected_error = IOError if sys.version_info < (3, 0) \
+        else FileNotFoundError
+    with self.assertRaises(expected_error):
+      in_toto_run(self.step_name, None, None, ["python", "--version"],
+          True, self.key, metadata_directory='nonexistentDir')
 
-class TestInTotoRecordStart(unittest.TestCase):
+  def test_not_a_directory(self):
+    """Fail run, passed metadata_directory is not a directory. """
+    fd, path = tempfile.mkstemp()
+    os.write(fd, b"hello in-toto")
+    os.close(fd)
+    # Windows will raise FileNotFoundError instead of NotADirectoryError
+    expected_error = IOError if sys.version_info < (3, 0) \
+        else (NotADirectoryError, FileNotFoundError)
+    with self.assertRaises(expected_error):
+      in_toto_run(self.step_name, None, None, ["python", "--version"],
+          True, self.key, metadata_directory=path)
+    os.remove(path)
+
+  @unittest.skipIf(os.name == 'nt', "chmod doesn't work properly on Windows")
+  def test_in_toto_read_only_metadata_directory(self):
+    """Fail run, passed metadata directory is read only"""
+    tmp_dir = os.path.realpath(tempfile.mkdtemp())
+    # make the directory read only
+    os.chmod(tmp_dir, stat.S_IREAD)
+    expected_error = IOError if sys.version_info < (3, 0) \
+        else PermissionError
+    with self.assertRaises(expected_error):
+      in_toto_run(self.step_name, None, None, ["python", "--version"],
+          True, self.key, metadata_directory=tmp_dir)
+    os.rmdir(tmp_dir)
+
+
+class TestInTotoRecordStart(unittest.TestCase, TmpDirMixin):
   """"Test in_toto_record_start(step_name, key, material_list). """
 
   @classmethod
   def setUpClass(self):
     """Create and change into temporary directory, generate key pair and dummy
     material, read key pair. """
-    self.working_dir = os.getcwd()
-
-    self.test_dir = tempfile.mkdtemp()
-    os.chdir(self.test_dir)
+    self.set_up_test_dir()
 
     self.key_path = "test_key"
-    generate_and_write_rsa_keypair(self.key_path)
-    self.key = prompt_import_rsa_key_from_file(self.key_path)
+    generate_and_write_unencrypted_rsa_keypair(self.key_path)
+    self.key = import_rsa_privatekey_from_file(self.key_path)
 
     self.step_name = "test_step"
     self.link_name_unfinished = UNFINISHED_FILENAME_FORMAT.format(step_name=self.step_name, keyid=self.key["keyid"])
@@ -588,9 +686,7 @@ class TestInTotoRecordStart(unittest.TestCase):
 
   @classmethod
   def tearDownClass(self):
-    """Change back to initial working dir and remove temp test directory. """
-    os.chdir(self.working_dir)
-    shutil.rmtree(self.test_dir)
+    self.tear_down_test_dir()
 
   def test_UNFINISHED_FILENAME_FORMAT(self):
     """Test if the unfinished filname format. """
@@ -620,24 +716,21 @@ class TestInTotoRecordStart(unittest.TestCase):
           self.step_name, [], signing_key=None, gpg_keyid=None,
           gpg_use_default=False)
 
-class TestInTotoRecordStop(unittest.TestCase):
+class TestInTotoRecordStop(unittest.TestCase, TmpDirMixin):
   """"Test in_toto_record_stop(step_name, key, product_list). """
 
   @classmethod
   def setUpClass(self):
     """Create and change into temporary directory, generate two key pairs
     and dummy product. """
-    self.working_dir = os.getcwd()
-
-    self.test_dir = tempfile.mkdtemp()
-    os.chdir(self.test_dir)
+    self.set_up_test_dir()
 
     self.key_path = "test-key"
     self.key_path2 = "test-key2"
-    generate_and_write_rsa_keypair(self.key_path)
-    generate_and_write_rsa_keypair(self.key_path2)
-    self.key = prompt_import_rsa_key_from_file(self.key_path)
-    self.key2 = prompt_import_rsa_key_from_file(self.key_path2)
+    generate_and_write_unencrypted_rsa_keypair(self.key_path)
+    generate_and_write_unencrypted_rsa_keypair(self.key_path2)
+    self.key = import_rsa_privatekey_from_file(self.key_path)
+    self.key2 = import_rsa_privatekey_from_file(self.key_path2)
 
     self.step_name = "test-step"
     self.link_name = "{}.{:.8}.link".format(self.step_name, self.key["keyid"])
@@ -649,9 +742,7 @@ class TestInTotoRecordStop(unittest.TestCase):
 
   @classmethod
   def tearDownClass(self):
-    """Change back to initial working dir and remove temp test directory. """
-    os.chdir(self.working_dir)
-    shutil.rmtree(self.test_dir)
+    self.tear_down_test_dir()
 
   def test_create_metadata_with_expected_product(self):
     """Test record stop records expected product. """
@@ -661,6 +752,23 @@ class TestInTotoRecordStop(unittest.TestCase):
     self.assertEqual(list(link.signed.products.keys()), [self.test_product])
     os.remove(self.link_name)
 
+  def test_compare_metadata_with_and_without_metadata_directory(self):
+    """Test record stop with and without metadata directory,
+     compare the expected product"""
+    tmp_dir = os.path.realpath(tempfile.mkdtemp(dir=os.getcwd()))
+    in_toto_record_start(self.step_name, [], self.key)
+    in_toto_record_stop(self.step_name, [self.test_product], self.key,
+        metadata_directory=tmp_dir)
+    link_path = os.path.join(tmp_dir, self.link_name)
+    link_with_md = Metablock.load(link_path)
+
+    in_toto_record_start(self.step_name, [], self.key)
+    in_toto_record_stop(self.step_name, [self.test_product], self.key)
+    link_without_md = Metablock.load(self.link_name)
+    self.assertEqual(link_with_md.signed, link_without_md.signed)
+    os.remove(link_path)
+    os.remove(self.link_name)
+
   def test_create_metadata_with_expected_cwd(self):
     """Test record start/stop run, verify cwd. """
     in_toto_record_start(self.step_name, [], self.key, record_environment=True)
@@ -745,6 +853,43 @@ class TestInTotoRecordStop(unittest.TestCase):
       for path in paths:
         os.remove(path)
 
+  def test_nonexistent_directory(self):
+    """Test record stop with nonexistent metadata directory"""
+    expected_error = IOError if sys.version_info < (3, 0) \
+        else FileNotFoundError
+    with self.assertRaises(expected_error):
+      in_toto_record_start(self.step_name, [], self.key)
+      in_toto_record_stop(self.step_name, [], self.key,
+          metadata_directory='nonexistentDir')
+
+  def test_not_a_directory(self):
+    """Test record stop, passed metadata directory is not a dir"""
+    fd, path = tempfile.mkstemp()
+    os.write(fd, b"hello in-toto")
+    os.close(fd)
+    # Windows will raise FileNotFoundError instead of NotADirectoryError
+    expected_error = IOError if sys.version_info < (3, 0) \
+        else (NotADirectoryError, FileNotFoundError)
+    with self.assertRaises(expected_error):
+      in_toto_record_start(self.step_name, [], self.key)
+      in_toto_record_stop(self.step_name, [], self.key,
+          metadata_directory=path)
+    os.remove(path)
+
+  @unittest.skipIf(os.name == 'nt', "chmod doesn't work properly on Windows")
+  def test_read_only_metadata_directory(self):
+    """Test record stop with read only metadata directory"""
+    tmp_dir = os.path.realpath(tempfile.mkdtemp())
+    # make the directory read only
+    os.chmod(tmp_dir, stat.S_IREAD)
+    expected_error = IOError if sys.version_info < (3, 0) \
+       else PermissionError
+    with self.assertRaises(expected_error):
+      in_toto_record_start(self.step_name, [], self.key)
+      in_toto_record_stop(self.step_name, [], self.key,
+          metadata_directory=tmp_dir)
+    os.rmdir(tmp_dir)
+
 
 if __name__ == "__main__":
   unittest.main()
